=== modified file 'drizzled/cursor.cc'
--- drizzled/cursor.cc	2010-06-04 20:14:31 +0000
+++ drizzled/cursor.cc	2010-06-07 06:04:12 +0000
@@ -192,13 +192,6 @@
   return session->getEngineData(engine);
 }
 
-Session *Cursor::ha_session(void) const
-{
-  assert(!table || !table->in_use || table->in_use == current_session);
-  return (table && table->in_use) ? table->in_use : current_session;
-}
-
-
 bool Cursor::is_fatal_error(int error, uint32_t flags)
 {
   if (!error ||
@@ -652,7 +645,17 @@
 void
 Cursor::setTransactionReadWrite()
 {
-  ResourceContext *resource_context= ha_session()->getResourceContext(engine);
+  ResourceContext *resource_context;
+
+  /*
+   * If the cursor has not context for execution then there should be no
+   * possible resource to gain (and if there is... then there is a bug such
+   * that in_use should have been set.
+ */
+  if (not table || not table->in_use)
+    return;
+
+  resource_context= table->in_use->getResourceContext(engine);
   /*
     When a storage engine method is called, the transaction must
     have been started, unless it's a DDL call, for which the

=== modified file 'drizzled/cursor.h'
--- drizzled/cursor.h	2010-06-03 09:06:27 +0000
+++ drizzled/cursor.h	2010-06-03 21:53:48 +0000
@@ -520,7 +520,6 @@
   /* Service methods for use by storage engines. */
   void ha_statistic_increment(ulong system_status_var::*offset) const;
   void **ha_data(Session *) const;
-  Session *ha_session(void) const;
 
 private:
   /* Private helpers */

=== modified file 'drizzled/drizzled.cc'
--- drizzled/drizzled.cc	2010-06-03 16:59:28 +0000
+++ drizzled/drizzled.cc	2010-06-04 02:43:34 +0000
@@ -1677,7 +1677,7 @@
   /* Don't print warnings for --loose options during bootstrap */
   if (level == ERROR_LEVEL || global_system_variables.log_warnings)
   {
-    plugin::ErrorMessage::vprintf(current_session, ERROR_LEVEL, format, args);
+    plugin::ErrorMessage::vprintf(NULL, ERROR_LEVEL, format, args);
   }
   va_end(args);
 }

=== modified file 'drizzled/dtcollation.cc'
--- drizzled/dtcollation.cc	2010-02-04 08:14:46 +0000
+++ drizzled/dtcollation.cc	2010-06-04 00:00:32 +0000
@@ -232,72 +232,10 @@
                        Item **args, uint32_t nargs, uint32_t flags,
                        int item_sep)
 {
-  Item **arg, *safe_args[2];
-
-  memset(safe_args, 0, sizeof(safe_args));
-
   if (agg_item_collations(coll, fname, args, nargs, flags, item_sep))
     return true;
 
-  /*
-    For better error reporting: save the first and the second argument.
-    We need this only if the the number of args is 3 or 2:
-    - for a longer argument list, "Illegal mix of collations"
-      doesn't display each argument's characteristics.
-    - if nargs is 1, then this error cannot happen.
-  */
-  if (nargs >=2 && nargs <= 3)
-  {
-    safe_args[0]= args[0];
-    safe_args[1]= args[item_sep];
-  }
-
-  Session *session= current_session;
-  bool res= false;
-  uint32_t i;
-
-  for (i= 0, arg= args; i < nargs; i++, arg+= item_sep)
-  {
-    Item* conv;
-    uint32_t dummy_offset;
-    if (!String::needs_conversion(0, (*arg)->collation.collation,
-                                  coll.collation,
-                                  &dummy_offset))
-      continue;
-
-    if (!(conv= (*arg)->safe_charset_converter(coll.collation)))
-    {
-      if (nargs >=2 && nargs <= 3)
-      {
-        /* restore the original arguments for better error message */
-        args[0]= safe_args[0];
-        args[item_sep]= safe_args[1];
-      }
-      my_coll_agg_error(args, nargs, fname, item_sep);
-      res= true;
-      break; // we cannot return here, we need to restore "arena".
-    }
-    if ((*arg)->type() == Item::FIELD_ITEM)
-      ((Item_field *)(*arg))->no_const_subst= 1;
-    /*
-      If in statement prepare, then we create a converter for two
-      constant items, do it once and then reuse it.
-      If we're in execution of a prepared statement, arena is NULL,
-      and the conv was created in runtime memory. This can be
-      the case only if the argument is a parameter marker ('?'),
-      because for all true constants the charset converter has already
-      been created in prepare. In this case register the change for
-      rollback.
-    */
-    session->change_item_tree(arg, conv);
-    /*
-      We do not check conv->fixed, because Item_func_conv_charset which can
-      be return by safe_charset_converter can't be fixed at creation
-    */
-    conv->fix_fields(session, arg);
-  }
-
-  return res;
+  return false;
 }
 
 

=== modified file 'drizzled/dtcollation.h'
--- drizzled/dtcollation.h	2010-02-04 08:14:46 +0000
+++ drizzled/dtcollation.h	2010-06-04 00:00:32 +0000
@@ -94,7 +94,10 @@
                                         Item **items, uint32_t nitems,
                                         uint32_t flags);
 
-/**
+/*
+
+ @note In Drizzle we have just one charset, so no conversion is required (though collation may).
+
   Collect arguments' character sets together.
 
   We allow to apply automatic character set conversion in some cases.

=== modified file 'drizzled/errmsg_print.cc'
--- drizzled/errmsg_print.cc	2010-05-15 18:23:34 +0000
+++ drizzled/errmsg_print.cc	2010-06-04 04:33:21 +0000
@@ -25,7 +25,6 @@
 #include "config.h"
 
 #include <drizzled/errmsg_print.h>
-#include <drizzled/current_session.h>
 #include <drizzled/plugin/error_message.h>
 
 #include <cerrno>
@@ -45,7 +44,7 @@
   bool rv;
   va_list args;
   va_start(args, format);
-  rv= plugin::ErrorMessage::vprintf(current_session, priority, format, args);
+  rv= plugin::ErrorMessage::vprintf(NULL, priority, format, args);
   va_end(args);
   return rv;
 }

=== modified file 'drizzled/function/math/divide.cc'
--- drizzled/function/math/divide.cc	2010-02-04 08:14:46 +0000
+++ drizzled/function/math/divide.cc	2010-06-04 04:13:08 +0000
@@ -87,7 +87,7 @@
 
 void Item_func_div::fix_length_and_dec()
 {
-  prec_increment= current_session->variables.div_precincrement;
+  prec_increment= session->variables.div_precincrement;
   Item_num_op::fix_length_and_dec();
 
   switch(hybrid_type)

=== modified file 'drizzled/function/math/divide.h'
--- drizzled/function/math/divide.h	2010-02-04 08:14:46 +0000
+++ drizzled/function/math/divide.h	2010-06-04 04:13:08 +0000
@@ -28,9 +28,10 @@
 
 class Item_func_div :public Item_num_op
 {
+  Session *session;
 public:
   uint32_t prec_increment;
-  Item_func_div(Item *a,Item *b) :Item_num_op(a,b) {}
+  Item_func_div(Session *arg, Item *a,Item *b) :Item_num_op(a,b), session(arg) {}
   int64_t int_op() { assert(0); return 0; }
   double real_op();
   my_decimal *decimal_op(my_decimal *);

=== modified file 'drizzled/item.cc'
--- drizzled/item.cc	2010-05-20 06:34:25 +0000
+++ drizzled/item.cc	2010-06-03 22:51:07 +0000
@@ -953,7 +953,8 @@
     Search for a column or derived column named as 'ref' in the SELECT
     clause of the current select.
   */
-  if (!(select_ref= find_item_in_list(ref, *(select->get_item_list()),
+  if (!(select_ref= find_item_in_list(session,
+                                      ref, *(select->get_item_list()),
                                       &counter, REPORT_EXCEPT_NOT_FOUND,
                                       &resolution)))
     return NULL; /* Some error occurred. */

=== modified file 'drizzled/item/cmpfunc.cc'
--- drizzled/item/cmpfunc.cc	2010-02-04 08:14:46 +0000
+++ drizzled/item/cmpfunc.cc	2010-06-04 04:13:08 +0000
@@ -721,9 +721,11 @@
   }
 
   if (error > 0)
+  {
     make_truncated_value_warning(session, DRIZZLE_ERROR::WARN_LEVEL_WARN,
                                  str->ptr(), str->length(),
                                  warn_type, warn_name);
+  }
 
   return value;
 }

=== modified file 'drizzled/item/create.cc'
--- drizzled/item/create.cc	2010-03-18 12:01:34 +0000
+++ drizzled/item/create.cc	2010-06-07 06:04:12 +0000
@@ -1687,7 +1687,7 @@
 {
   Item *i1= new (session->mem_root) Item_int((char*) "1", 1, 1);
   Item *i2= new (session->mem_root) Item_func_tan(arg1);
-  return new (session->mem_root) Item_func_div(i1, i2);
+  return new (session->mem_root) Item_func_div(session, i1, i2);
 }
 
 Create_func_date_format Create_func_date_format::s_singleton;

=== modified file 'drizzled/item/field.cc'
--- drizzled/item/field.cc	2010-05-20 06:34:25 +0000
+++ drizzled/item/field.cc	2010-06-03 22:51:07 +0000
@@ -789,7 +789,8 @@
       {
         uint32_t counter;
         enum_resolution_type resolution;
-        Item** res= find_item_in_list(this, session->lex->current_select->item_list,
+        Item** res= find_item_in_list(session,
+                                      this, session->lex->current_select->item_list,
                                       &counter, REPORT_EXCEPT_NOT_FOUND,
                                       &resolution);
         if (!res)

=== modified file 'drizzled/plugin/error_message.cc'
--- drizzled/plugin/error_message.cc	2010-05-15 18:23:34 +0000
+++ drizzled/plugin/error_message.cc	2010-06-04 02:43:34 +0000
@@ -90,7 +90,7 @@
 
 
 bool plugin::ErrorMessage::vprintf(Session *session, int priority,
-                                 char const *format, va_list ap)
+                                   char const *format, va_list ap)
 {
 
   /* check to see if any errmsg plugin has been loaded

=== modified file 'drizzled/plugin/storage_engine.cc'
--- drizzled/plugin/storage_engine.cc	2010-05-20 06:34:25 +0000
+++ drizzled/plugin/storage_engine.cc	2010-06-04 02:25:28 +0000
@@ -823,15 +823,8 @@
     {
       const char *err_msg= ER(ER_DUP_ENTRY_WITH_KEY_NAME);
 
-      if (key_nr == 0 &&
-          (table->key_info[0].key_part[0].field->flags &
-           AUTO_INCREMENT_FLAG)
-          && (current_session)->lex->sql_command == SQLCOM_ALTER_TABLE)
-      {
-        err_msg= ER(ER_DUP_ENTRY_AUTOINCREMENT_CASE);
-      }
-
       print_keydup_error(key_nr, err_msg, *table);
+
       return;
     }
     textno=ER_DUP_KEY;

=== modified file 'drizzled/sql_base.cc'
--- drizzled/sql_base.cc	2010-06-04 23:50:43 +0000
+++ drizzled/sql_base.cc	2010-06-07 06:04:12 +0000
@@ -2942,7 +2942,8 @@
 
 
 Item **
-find_item_in_list(Item *find, List<Item> &items, uint32_t *counter,
+find_item_in_list(Session *session,
+                  Item *find, List<Item> &items, uint32_t *counter,
                   find_item_error_report_type report_error,
                   enum_resolution_type *resolution)
 {
@@ -3022,7 +3023,7 @@
             */
             if (report_error != IGNORE_ERRORS)
               my_error(ER_NON_UNIQ_ERROR, MYF(0),
-                       find->full_name(), current_session->where);
+                       find->full_name(), session->where);
             return (Item**) 0;
           }
           found_unaliased= li.ref();
@@ -3053,7 +3054,7 @@
               continue;                           // Same field twice
             if (report_error != IGNORE_ERRORS)
               my_error(ER_NON_UNIQ_ERROR, MYF(0),
-                       find->full_name(), current_session->where);
+                       find->full_name(), session->where);
             return (Item**) 0;
           }
           found= li.ref();
@@ -3105,7 +3106,7 @@
     {
       if (report_error != IGNORE_ERRORS)
         my_error(ER_NON_UNIQ_ERROR, MYF(0),
-                 find->full_name(), current_session->where);
+                 find->full_name(), session->where);
       return (Item **) 0;
     }
     if (found_unaliased)
@@ -3121,7 +3122,7 @@
   {
     if (report_error == REPORT_ALL_ERRORS)
       my_error(ER_BAD_FIELD_ERROR, MYF(0),
-               find->full_name(), current_session->where);
+               find->full_name(), session->where);
     return (Item **) 0;
   }
   else
@@ -3442,7 +3443,7 @@
 */
 
 static bool
-store_natural_using_join_columns(Session *,
+store_natural_using_join_columns(Session *session,
                                  TableList *natural_using_join,
                                  TableList *table_ref_1,
                                  TableList *table_ref_2,
@@ -3496,7 +3497,7 @@
         if (!(common_field= it++))
         {
           my_error(ER_BAD_FIELD_ERROR, MYF(0), using_field_name_ptr,
-                   current_session->where);
+                   session->where);
           goto err;
         }
         if (!my_strcasecmp(system_charset_info,

=== modified file 'drizzled/sql_base.h'
--- drizzled/sql_base.h	2010-03-31 05:53:34 +0000
+++ drizzled/sql_base.h	2010-06-03 22:51:07 +0000
@@ -89,7 +89,8 @@
   RESOLVED_WITH_NO_ALIAS,
   RESOLVED_AGAINST_ALIAS
 };
-Item ** find_item_in_list(Item *item, List<Item> &items, uint32_t *counter,
+Item ** find_item_in_list(Session *session,
+                          Item *item, List<Item> &items, uint32_t *counter,
                           find_item_error_report_type report_error,
                           enum_resolution_type *resolution);
 bool insert_fields(Session *session, Name_resolution_context *context,

=== modified file 'drizzled/sql_select.cc'
--- drizzled/sql_select.cc	2010-06-04 23:50:43 +0000
+++ drizzled/sql_select.cc	2010-06-07 06:04:12 +0000
@@ -5666,7 +5666,7 @@
     return false;
   }
   /* Lookup the current GROUP/order_st field in the SELECT clause. */
-  select_item= find_item_in_list(order_item, fields, &counter,
+  select_item= find_item_in_list(session, order_item, fields, &counter,
                                  REPORT_EXCEPT_NOT_FOUND, &resolution);
   if (!select_item)
     return true; /* The item is not unique, or some other error occured. */

=== modified file 'drizzled/sql_yacc.yy'
--- drizzled/sql_yacc.yy	2010-05-21 16:22:35 +0000
+++ drizzled/sql_yacc.yy	2010-06-04 21:03:29 +0000
@@ -67,7 +67,8 @@
 #define DRIZZLE_YYABORT_UNLESS(A)         \
   if (!(A))                             \
   {                                     \
-    my_parse_error(ER(ER_SYNTAX_ERROR));\
+    struct my_parse_error_st pass= { ER(ER_SYNTAX_ERROR), YYSession };\
+    my_parse_error(&pass);\
     DRIZZLE_YYABORT;                      \
   }
 
@@ -102,9 +103,18 @@
   parser.
 */
 
-static void my_parse_error(const char *s)
+struct my_parse_error_st {
+  const char *s;
+  Session *session;
+};
+
+static void my_parse_error(void *arg)
 {
-  Session *session= current_session;
+ struct my_parse_error_st *ptr= (struct my_parse_error_st *)arg;
+
+  const char *s= ptr->s;
+  Session *session= ptr->session;
+
   Lex_input_stream *lip= session->m_lip;
 
   const char *yytext= lip->get_tok_start();
@@ -148,7 +158,9 @@
   /* "parse error" changed into "syntax error" between bison 1.75 and 1.875 */
   if (strcmp(s,"parse error") == 0 || strcmp(s,"syntax error") == 0)
     s= ER(ER_SYNTAX_ERROR);
-  my_parse_error(s);
+
+  struct my_parse_error_st pass= { s, session };
+  my_parse_error(&pass);
 }
 
 /**
@@ -240,7 +252,7 @@
    @return <code>false</code> if successful, <code>true</code> if an error was
    reported. In the latter case parsing should stop.
  */
-static bool add_select_to_union_list(LEX *lex, bool is_union_distinct)
+static bool add_select_to_union_list(Session *session, LEX *lex, bool is_union_distinct)
 {
   if (lex->result)
   {
@@ -250,7 +262,8 @@
   }
   if (lex->current_select->linkage == GLOBAL_OPTIONS_TYPE)
   {
-    my_parse_error(ER(ER_SYNTAX_ERROR));
+    struct my_parse_error_st pass= { ER(ER_SYNTAX_ERROR), session };
+    my_parse_error(&pass);
     return true;
   }
   /* This counter shouldn't be incremented for UNION parts */
@@ -272,12 +285,13 @@
    @return false if successful, true if an error was reported. In the latter
    case parsing should stop.
  */
-static bool setup_select_in_parentheses(LEX *lex)
+static bool setup_select_in_parentheses(Session *session, LEX *lex)
 {
   Select_Lex * sel= lex->current_select;
   if (sel->set_braces(1))
   {
-    my_parse_error(ER(ER_SYNTAX_ERROR));
+    struct my_parse_error_st pass= { ER(ER_SYNTAX_ERROR), session };
+    my_parse_error(&pass);
     return true;
   }
   if (sel->linkage == UNION_TYPE &&
@@ -285,7 +299,8 @@
       sel->master_unit()->first_select()->linkage ==
       UNION_TYPE)
   {
-    my_parse_error(ER(ER_SYNTAX_ERROR));
+    struct my_parse_error_st pass= { ER(ER_SYNTAX_ERROR), session };
+    my_parse_error(&pass);
     return true;
   }
   if (sel->linkage == UNION_TYPE &&
@@ -302,14 +317,14 @@
   return false;
 }
 
-static Item* reserved_keyword_function(const std::string &name, List<Item> *item_list)
+static Item* reserved_keyword_function(Session *session, const std::string &name, List<Item> *item_list)
 {
   const plugin::Function *udf= plugin::Function::get(name.c_str(), name.length());
   Item *item= NULL;
 
   if (udf)
   {
-    item= Create_udf_func::s_singleton.create(current_session, udf, item_list);
+    item= Create_udf_func::s_singleton.create(session, udf, item_list);
   } else {
     my_error(ER_SP_DOES_NOT_EXIST, MYF(0), "FUNCTION", name.c_str());
   }
@@ -2479,7 +2494,7 @@
 select_paren:
           SELECT_SYM select_part2
           {
-            if (setup_select_in_parentheses(Lex))
+            if (setup_select_in_parentheses(YYSession, Lex))
               DRIZZLE_YYABORT;
           }
         | '(' select_paren ')'
@@ -2489,7 +2504,7 @@
 select_paren_derived:
           SELECT_SYM select_part2_derived
           {
-            if (setup_select_in_parentheses(Lex))
+            if (setup_select_in_parentheses(YYSession, Lex))
               DRIZZLE_YYABORT;
           }
         | '(' select_paren_derived ')'
@@ -2502,13 +2517,15 @@
             Select_Lex * sel= lex->current_select;
             if (lex->current_select->set_braces(0))
             {
-              my_parse_error(ER(ER_SYNTAX_ERROR));
+              struct my_parse_error_st pass= { ER(ER_SYNTAX_ERROR), YYSession };
+              my_parse_error(&pass);
               DRIZZLE_YYABORT;
             }
             if (sel->linkage == UNION_TYPE &&
                 sel->master_unit()->first_select()->braces)
             {
-              my_parse_error(ER(ER_SYNTAX_ERROR));
+              struct my_parse_error_st pass= { ER(ER_SYNTAX_ERROR), YYSession };
+              my_parse_error(&pass);
               DRIZZLE_YYABORT;
             }
           }
@@ -2859,7 +2876,7 @@
         | bit_expr '*' bit_expr %prec '*'
           { $$= new Item_func_mul($1,$3); }
         | bit_expr '/' bit_expr %prec '/'
-          { $$= new Item_func_div($1,$3); }
+          { $$= new Item_func_div(YYSession,$1,$3); }
         | bit_expr '%' bit_expr %prec '%'
           { $$= new Item_func_mod($1,$3); }
         | bit_expr DIV_SYM bit_expr %prec DIV_SYM
@@ -2984,7 +3001,7 @@
         | CURRENT_USER optional_braces
           {
             std::string user_str("user");
-            if (! ($$= reserved_keyword_function(user_str, NULL)))
+            if (! ($$= reserved_keyword_function(YYSession, user_str, NULL)))
             {
               DRIZZLE_YYABORT;
             }
@@ -3045,7 +3062,7 @@
         | USER '(' ')'
           {
             std::string user_str("user");
-            if (! ($$= reserved_keyword_function(user_str, NULL)))
+            if (! ($$= reserved_keyword_function(YYSession, user_str, NULL)))
             {
               DRIZZLE_YYABORT;
             }
@@ -3107,7 +3124,7 @@
             args->push_back($3);
             args->push_back($5);
             args->push_back($7);
-            if (! ($$= reserved_keyword_function(reverse_str, args)))
+            if (! ($$= reserved_keyword_function(YYSession, reverse_str, args)))
             {
               DRIZZLE_YYABORT;
             }
@@ -3118,7 +3135,7 @@
             List<Item> *args= new (YYSession->mem_root) List<Item>;
             args->push_back($3);
             args->push_back($5);
-            if (! ($$= reserved_keyword_function(reverse_str, args)))
+            if (! ($$= reserved_keyword_function(YYSession, reverse_str, args)))
             {
               DRIZZLE_YYABORT;
             }
@@ -3130,7 +3147,7 @@
             args->push_back($3);
             args->push_back($5);
             args->push_back($7);
-            if (! ($$= reserved_keyword_function(reverse_str, args)))
+            if (! ($$= reserved_keyword_function(YYSession, reverse_str, args)))
             {
               DRIZZLE_YYABORT;
             }
@@ -3141,7 +3158,7 @@
             List<Item> *args= new (YYSession->mem_root) List<Item>;
             args->push_back($3);
             args->push_back($5);
-            if (! ($$= reserved_keyword_function(reverse_str, args)))
+            if (! ($$= reserved_keyword_function(YYSession, reverse_str, args)))
             {
               DRIZZLE_YYABORT;
             }
@@ -3177,7 +3194,7 @@
         | DATABASE '(' ')'
           {
             std::string database_str("database");
-            if (! ($$= reserved_keyword_function(database_str, NULL)))
+            if (! ($$= reserved_keyword_function(YYSession, database_str, NULL)))
             {
               DRIZZLE_YYABORT;
             }
@@ -3199,7 +3216,7 @@
             std::string reverse_str("reverse");
             List<Item> *args= new (YYSession->mem_root) List<Item>;
             args->push_back($3);
-            if (! ($$= reserved_keyword_function(reverse_str, args)))
+            if (! ($$= reserved_keyword_function(YYSession, reverse_str, args)))
             {
               DRIZZLE_YYABORT;
             }
@@ -3379,7 +3396,8 @@
             /* disallow "SELECT @@global.global.variable" */
             if ($3.str && $4.str && check_reserved_words(&$3))
             {
-              my_parse_error(ER(ER_SYNTAX_ERROR));
+              struct my_parse_error_st pass= { ER(ER_SYNTAX_ERROR), YYSession };
+              my_parse_error(&pass);
               DRIZZLE_YYABORT;
             }
             if (!($$= get_system_var(YYSession, $2, $3, $4)))
@@ -3421,7 +3439,8 @@
             LEX *lex= Lex;
             if (lex->current_select->inc_in_sum_expr())
             {
-              my_parse_error(ER(ER_SYNTAX_ERROR));
+              struct my_parse_error_st pass= { ER(ER_SYNTAX_ERROR), YYSession };
+              my_parse_error(&pass);
               DRIZZLE_YYABORT;
             }
           }
@@ -3695,7 +3714,8 @@
             {
               if (sel->set_braces(1))
               {
-                my_parse_error(ER(ER_SYNTAX_ERROR));
+                struct my_parse_error_st pass= { ER(ER_SYNTAX_ERROR), YYSession };
+                my_parse_error(&pass);
                 DRIZZLE_YYABORT;
               }
               /* select in braces, can't contain global parameters */
@@ -3760,7 +3780,8 @@
             else if (($3->select_lex && $3->select_lex->master_unit()->is_union()) || $5)
             {
               /* simple nested joins cannot have aliases or unions */
-              my_parse_error(ER(ER_SYNTAX_ERROR));
+              struct my_parse_error_st pass= { ER(ER_SYNTAX_ERROR), YYSession };
+              my_parse_error(&pass);
               DRIZZLE_YYABORT;
             }
             else
@@ -3774,7 +3795,7 @@
           UNION_SYM
           union_option
           {
-            if (add_select_to_union_list(Lex, (bool)$3))
+            if (add_select_to_union_list(YYSession, Lex, (bool)$3))
               DRIZZLE_YYABORT;
           }
           query_specification
@@ -3796,13 +3817,15 @@
             Select_Lex * sel= lex->current_select;
             if (lex->current_select->set_braces(0))
             {
-              my_parse_error(ER(ER_SYNTAX_ERROR));
+              struct my_parse_error_st pass= { ER(ER_SYNTAX_ERROR), YYSession };
+              my_parse_error(&pass);
               DRIZZLE_YYABORT;
             }
             if (sel->linkage == UNION_TYPE &&
                 sel->master_unit()->first_select()->braces)
             {
-              my_parse_error(ER(ER_SYNTAX_ERROR));
+              struct my_parse_error_st pass= { ER(ER_SYNTAX_ERROR), YYSession };
+              my_parse_error(&pass);
               DRIZZLE_YYABORT;
             }
           }
@@ -3842,7 +3865,8 @@
               DRIZZLE_YYABORT;
             if (!$3 && $$)
             {
-              my_parse_error(ER(ER_SYNTAX_ERROR));
+              struct my_parse_error_st pass= { ER(ER_SYNTAX_ERROR), YYSession };
+              my_parse_error(&pass);
               DRIZZLE_YYABORT;
             }
           }
@@ -3854,7 +3878,8 @@
             lex->derived_tables|= DERIVED_SUBQUERY;
             if (!lex->expr_allows_subselect)
             {
-              my_parse_error(ER(ER_SYNTAX_ERROR));
+              struct my_parse_error_st pass= { ER(ER_SYNTAX_ERROR), YYSession };
+              my_parse_error(&pass);
               DRIZZLE_YYABORT;
             }
             if (lex->current_select->linkage == GLOBAL_OPTIONS_TYPE ||
@@ -3885,7 +3910,8 @@
             if (!sel->embedding || sel->end_nested_join(lex->session))
             {
               /* we are not in parentheses */
-              my_parse_error(ER(ER_SYNTAX_ERROR));
+              struct my_parse_error_st pass= { ER(ER_SYNTAX_ERROR), YYSession };
+              my_parse_error(&pass);
               DRIZZLE_YYABORT;
             }
             embedding= Lex->current_select->embedding;
@@ -6201,7 +6227,7 @@
 union_list:
           UNION_SYM union_option
           {
-            if (add_select_to_union_list(Lex, (bool)$2))
+            if (add_select_to_union_list(YYSession, Lex, (bool)$2))
               DRIZZLE_YYABORT;
           }
           select_init
@@ -6271,7 +6297,7 @@
         | query_expression_body
           UNION_SYM union_option
           {
-            if (add_select_to_union_list(Lex, (bool)$3))
+            if (add_select_to_union_list(YYSession, Lex, (bool)$3))
               DRIZZLE_YYABORT;
           }
           query_specification
@@ -6294,7 +6320,8 @@
             LEX *lex=Lex;
             if (!lex->expr_allows_subselect)
             {
-              my_parse_error(ER(ER_SYNTAX_ERROR));
+              struct my_parse_error_st pass= { ER(ER_SYNTAX_ERROR), YYSession };
+              my_parse_error(&pass);
               DRIZZLE_YYABORT;
             }
             /*

=== modified file 'drizzled/table.cc'
--- drizzled/table.cc	2010-06-04 23:50:43 +0000
+++ drizzled/table.cc	2010-06-07 06:04:12 +0000
@@ -919,6 +919,7 @@
   table->db_stat=HA_OPEN_KEYFILE+HA_OPEN_RNDFILE;
   table->map=1;
   table->copy_blobs= 1;
+  assert(session);
   table->in_use= session;
   table->quick_keys.reset();
   table->covering_keys.reset();
@@ -1435,6 +1436,7 @@
 				       &param->recinfo, select_options))
       goto err;
   }
+  assert(table->in_use);
   if (table->open_tmp_table())
     goto err;
 

=== modified file 'plugin/embedded_innodb/embedded_innodb_engine.cc'
--- plugin/embedded_innodb/embedded_innodb_engine.cc	2010-06-05 00:15:57 +0000
+++ plugin/embedded_innodb/embedded_innodb_engine.cc	2010-06-07 06:04:12 +0000
@@ -380,7 +380,7 @@
 {
   uint64_t nr;
   ib_err_t err;
-  ib_trx_t transaction= *get_trx(ha_session());
+  ib_trx_t transaction= *get_trx(table->in_use);
   ib_cursor_attach_trx(cursor, transaction);
   tuple= ib_clust_read_tuple_create(cursor);
   err= ib_cursor_last(cursor);
@@ -1556,7 +1556,7 @@
   ib_err_t err;
   int ret= 0;
 
-  ib_trx_t transaction= *get_trx(ha_session());
+  ib_trx_t transaction= *get_trx(table->in_use);
 
   tuple= ib_clust_read_tuple_create(cursor);
 
@@ -1582,7 +1582,7 @@
     err= ib_cursor_reset(cursor);
     innodb_engine->doCommit(current_session, true);
     innodb_engine->doStartTransaction(current_session, START_TRANS_NO_OPTIONS);
-    transaction= *get_trx(ha_session());
+    transaction= *get_trx(table->in_use);
     assert(err == DB_SUCCESS);
     ib_cursor_attach_trx(cursor, transaction);
     err= ib_cursor_first(cursor);
@@ -1718,7 +1718,7 @@
      so only support TRUNCATE and not DELETE FROM t;
      (this is what ha_innodb does)
   */
-  if (session_sql_command(ha_session()) != SQLCOM_TRUNCATE)
+  if (session_sql_command(table->in_use) != SQLCOM_TRUNCATE)
     return HA_ERR_WRONG_COMMAND;
 
   ib_id_t id;
@@ -1733,7 +1733,7 @@
   {
     ib_err_t rollback_err= ib_trx_rollback(transaction);
 
-    push_warning_printf(ha_session(), DRIZZLE_ERROR::WARN_LEVEL_ERROR,
+    push_warning_printf(table->in_use, DRIZZLE_ERROR::WARN_LEVEL_ERROR,
                         ER_CANT_DELETE_FILE,
                         _("Cannot Lock Embedded InnoDB Data Dictionary. InnoDB Error %d (%s)\n"),
                         err, ib_strerror(err));
@@ -1773,7 +1773,7 @@
     doEndTableScan();
   in_table_scan= true;
 
-  transaction= *get_trx(ha_session());
+  transaction= *get_trx(table->in_use);
 
   assert(transaction != NULL);
 
@@ -2021,7 +2021,7 @@
 
 int EmbeddedInnoDBCursor::doStartIndexScan(uint32_t keynr, bool)
 {
-  ib_trx_t transaction= *get_trx(ha_session());
+  ib_trx_t transaction= *get_trx(table->in_use);
 
   active_index= keynr;
 

=== modified file 'plugin/heap/ha_heap.cc'
--- plugin/heap/ha_heap.cc	2010-06-05 00:14:39 +0000
+++ plugin/heap/ha_heap.cc	2010-06-07 06:04:12 +0000
@@ -199,7 +199,7 @@
     HP_SHARE *internal_share= NULL;
     message::Table create_proto;
 
-    if (!heap_storage_engine->heap_create_table(ha_session(), name, table,
+    if (!heap_storage_engine->heap_create_table(table->in_use, name, table,
                                                 internal_table,
                                                 create_proto,
                                                 &internal_share))

=== modified file 'plugin/innobase/handler/ha_innodb.cc'
--- plugin/innobase/handler/ha_innodb.cc	2010-06-04 23:50:43 +0000
+++ plugin/innobase/handler/ha_innodb.cc	2010-06-07 06:04:12 +0000
@@ -1638,6 +1638,7 @@
 {
   trx_t*    trx;
 
+  assert(session);
   trx = check_trx_exists(session);
 
   if (prebuilt->trx != trx) {
@@ -1648,20 +1649,6 @@
   user_session = session;
 }
 
-/*********************************************************************//**
-Updates the user_thd field in a handle and also allocates a new InnoDB
-transaction handle if needed, and updates the transaction fields in the
-prebuilt struct. */
-UNIV_INTERN
-void
-ha_innobase::update_session()
-/*=====================*/
-{
-  Session*  session = ha_session();
-  ut_ad(EQ_CURRENT_SESSION(session));
-  update_session(session);
-}
-
 /*****************************************************************//**
 Convert an SQL identifier to the MySQL system_charset_info (UTF-8)
 and quote it if needed.
@@ -2747,7 +2734,7 @@
   UT_NOT_USED(mode);
   UT_NOT_USED(test_if_locked);
 
-  session = ha_session();
+  session= table->in_use;
 
   /* Under some cases Drizzle seems to call this function while
   holding btr_search_latch. This breaks the latching order as
@@ -2954,7 +2941,7 @@
 {
   Session*  session;
 
-  session = ha_session();
+  session= table->in_use;
   if (session != NULL) {
     getTransactionalEngine()->releaseTemporaryLatches(session);
   }
@@ -4390,7 +4377,7 @@
 ha_innobase::try_semi_consistent_read(bool yes)
 /*===========================================*/
 {
-  ut_a(prebuilt->trx == session_to_trx(ha_session()));
+  ut_a(prebuilt->trx == session_to_trx(table->in_use));
 
   /* Row read type is set to semi consistent read if this was
   requested by the MySQL and either innodb_locks_unsafe_for_binlog
@@ -4692,7 +4679,7 @@
 
   ha_statistic_increment(&system_status_var::ha_read_key_count);
 
-  ut_ad(user_session == ha_session());
+  ut_ad(user_session == table->in_use);
   ut_a(prebuilt->trx == session_to_trx(user_session));
 
   if (keynr != MAX_KEY && table->getShare()->sizeKeys() > 0) 
@@ -4725,7 +4712,7 @@
       index, even if it was internally generated by
       InnoDB */
 {
-  ut_ad(user_session == ha_session());
+  ut_ad(user_session == table->in_use);
   ut_a(prebuilt->trx == session_to_trx(user_session));
 
   active_index = keynr;
@@ -5031,7 +5018,7 @@
 
   ha_statistic_increment(&system_status_var::ha_read_rnd_count);
 
-  ut_a(prebuilt->trx == session_to_trx(ha_session()));
+  ut_a(prebuilt->trx == session_to_trx(table->in_use));
 
   if (prebuilt->clust_index_was_generated) {
     /* No primary key was defined for the table and we
@@ -5077,7 +5064,7 @@
 {
   uint    len;
 
-  ut_a(prebuilt->trx == session_to_trx(ha_session()));
+  ut_a(prebuilt->trx == session_to_trx(table->in_use));
 
   if (prebuilt->clust_index_was_generated) {
     /* No primary key was defined for the table and we
@@ -5694,7 +5681,7 @@
 
   ut_a(prebuilt->trx);
   ut_a(prebuilt->trx->magic_n == TRX_MAGIC_N);
-  ut_a(prebuilt->trx == session_to_trx(ha_session()));
+  ut_a(prebuilt->trx == session_to_trx(table->in_use));
 
   dict_table = prebuilt->table;
   trx = prebuilt->trx;
@@ -5723,7 +5710,7 @@
   /* Get the transaction associated with the current session, or create one
   if not yet created, and update prebuilt->trx */
 
-  update_session(ha_session());
+  update_session(table->in_use);
 
   if (session_sql_command(user_session) != SQLCOM_TRUNCATE) {
   fallback:
@@ -6059,7 +6046,7 @@
   ulint   mode2;
   mem_heap_t* heap;
 
-  ut_a(prebuilt->trx == session_to_trx(ha_session()));
+  ut_a(prebuilt->trx == session_to_trx(table->in_use));
 
   prebuilt->trx->op_info = (char*)"estimating records in index range";
 
@@ -6154,7 +6141,7 @@
   external_lock(). To be safe, update the session of the current table
   handle. */
 
-  update_session(ha_session());
+  update_session(table->in_use);
 
   prebuilt->trx->op_info = (char*)
          "calculating upper bound for table rows";
@@ -6277,7 +6264,7 @@
   external_lock(). To be safe, update the session of the current table
   handle. */
 
-  update_session(ha_session());
+  update_session(table->in_use);
 
   /* In case MySQL calls this in the middle of a SELECT query, release
   possible adaptive hash latch to avoid deadlocks of threads */
@@ -6388,7 +6375,8 @@
 
         Session*  session;
 
-        session = ha_session();
+        session= table->in_use;
+        assert(session);
 
         push_warning_printf(
           session,
@@ -6534,7 +6522,7 @@
 {
   ulint   ret;
 
-  assert(session == ha_session());
+  assert(session == table->in_use);
   ut_a(prebuilt->trx);
   ut_a(prebuilt->trx->magic_n == TRX_MAGIC_N);
   ut_a(prebuilt->trx == session_to_trx(session));
@@ -6578,7 +6566,7 @@
     return((char*)comment); /* string too long */
   }
 
-  update_session(ha_session());
+  update_session(table->in_use);
 
   prebuilt->trx->op_info = (char*)"returning table comment";
 
@@ -6649,7 +6637,7 @@
   external_lock(). To be safe, update the session of the current table
   handle. */
 
-  update_session(ha_session());
+  update_session(table->in_use);
 
   prebuilt->trx->op_info = (char*)"getting info on foreign keys";
 
@@ -6698,7 +6686,7 @@
   dict_foreign_t* foreign;
 
   ut_a(prebuilt != NULL);
-  update_session(ha_session());
+  update_session(table->in_use);
   prebuilt->trx->op_info = (char*)"getting list of foreign keys";
   trx_search_latch_release_if_reserved(prebuilt->trx);
   mutex_enter(&(dict_sys->mutex));
@@ -6836,7 +6824,7 @@
 {
   bool  can_switch;
 
-  ut_a(prebuilt->trx == session_to_trx(ha_session()));
+  ut_a(prebuilt->trx == session_to_trx(table->in_use));
 
   prebuilt->trx->op_info =
       "determining if there are foreign key constraints";
@@ -6924,16 +6912,16 @@
       either, because the calling threads may change.
       CAREFUL HERE, OR MEMORY CORRUPTION MAY OCCUR! */
     case HA_EXTRA_IGNORE_DUP_KEY:
-      session_to_trx(ha_session())->duplicates |= TRX_DUP_IGNORE;
+      session_to_trx(table->in_use)->duplicates |= TRX_DUP_IGNORE;
       break;
     case HA_EXTRA_WRITE_CAN_REPLACE:
-      session_to_trx(ha_session())->duplicates |= TRX_DUP_REPLACE;
+      session_to_trx(table->in_use)->duplicates |= TRX_DUP_REPLACE;
       break;
     case HA_EXTRA_WRITE_CANNOT_REPLACE:
-      session_to_trx(ha_session())->duplicates &= ~TRX_DUP_REPLACE;
+      session_to_trx(table->in_use)->duplicates &= ~TRX_DUP_REPLACE;
       break;
     case HA_EXTRA_NO_IGNORE_DUP_KEY:
-      session_to_trx(ha_session())->duplicates &=
+      session_to_trx(table->in_use)->duplicates &=
         ~(TRX_DUP_IGNORE | TRX_DUP_REPLACE);
       break;
     default:/* Do nothing */
@@ -7579,7 +7567,7 @@
   uint64_t  autoinc = 0;
 
   /* Prepare prebuilt->trx in the table handle */
-  update_session(ha_session());
+  update_session(table->in_use);
 
   error = innobase_get_autoinc(&autoinc);
 
@@ -7675,7 +7663,7 @@
 {
   int error;
 
-  update_session(ha_session());
+  update_session(table->in_use);
 
   error = row_lock_table_autoinc_for_mysql(prebuilt);
 

=== modified file 'plugin/innobase/handler/ha_innodb.h'
--- plugin/innobase/handler/ha_innodb.h	2010-05-22 00:29:14 +0000
+++ plugin/innobase/handler/ha_innodb.h	2010-06-07 06:04:12 +0000
@@ -85,7 +85,6 @@
 	UNIV_INTERN uint store_key_val_for_row(uint keynr, char* buff, 
                                    uint buff_len, const unsigned char* record);
 	UNIV_INTERN void update_session(Session* session);
-	UNIV_INTERN void update_session();
 	UNIV_INTERN int change_active_index(uint32_t keynr);
 	UNIV_INTERN int general_fetch(unsigned char* buf, uint32_t direction, uint32_t match_mode);
 	UNIV_INTERN ulint innobase_lock_autoinc();
@@ -188,11 +187,15 @@
 	UNIV_INTERN int cmp_ref(const unsigned char *ref1, const unsigned char *ref2);
 	/** Fast index creation (smart ALTER TABLE) @see handler0alter.cc @{ */
         // Don't use these, I have just left them in here as reference for
-        // the future. -Brian
-	UNIV_INTERN int add_index(TABLE *table_arg, KeyInfo *key_info, uint num_of_keys);
-	UNIV_INTERN int prepare_drop_index(TABLE *table_arg, uint *key_num,
+        // the future. -Brian 
+#if 0
+	UNIV_INTERN int add_index(Session *session, TABLE *table_arg, KeyInfo *key_info, uint num_of_keys);
+	UNIV_INTERN int prepare_drop_index(Session *session,
+                                           TABLE *table_arg,
+                                           uint *key_num,
                                            uint num_of_keys);
-        UNIV_INTERN int final_drop_index(TABLE *table_arg);
+        UNIV_INTERN int final_drop_index(Session *session, TABLE *table_arg);
+#endif
 	/** @} */
 public:
   int read_range_first(const key_range *start_key, const key_range *end_key,

=== modified file 'plugin/innobase/handler/handler0alter.cc'
--- plugin/innobase/handler/handler0alter.cc	2010-06-05 00:14:39 +0000
+++ plugin/innobase/handler/handler0alter.cc	2010-06-07 06:04:12 +0000
@@ -194,6 +194,7 @@
 	}
 }
 
+#if 0 // This is a part of the fast index code.
 /******************************************************************//**
 Removes the filename encoding of a database and table name. */
 static
@@ -221,6 +222,7 @@
 	}
 }
 
+
 /*******************************************************************//**
 This function checks that index keys are sensible.
 @return	0 or error number */
@@ -484,6 +486,7 @@
 
 
 @return	key definitions or NULL */
+
 static
 merge_index_def_t*
 innobase_create_key_def(
@@ -596,6 +599,7 @@
 	return(name);
 }
 
+
 /*******************************************************************//**
 Create indexes.
 @return	0 or error number */
@@ -603,6 +607,7 @@
 int
 ha_innobase::add_index(
 /*===================*/
+                       Session *session,
 	Table*	i_table,	/*!< in: Table where indexes are created */
 	KeyInfo*	key_info,	/*!< in: Indexes to be created */
 	uint	num_of_keys)	/*!< in: Number of indexes to be created */
@@ -627,7 +632,7 @@
 		return(HA_ERR_WRONG_COMMAND);
 	}
 
-	update_session();
+	update_session(session);
 
 	heap = mem_heap_create(1024);
 
@@ -907,6 +912,7 @@
 int
 ha_innobase::prepare_drop_index(
 /*============================*/
+                                Session *session,
 	Table*	i_table,	/*!< in: Table where indexes are dropped */
 	uint*	key_num,	/*!< in: Key nums to be dropped */
 	uint	num_of_keys)	/*!< in: Number of keys to be dropped */
@@ -922,7 +928,7 @@
 		return(HA_ERR_WRONG_COMMAND);
 	}
 
-	update_session();
+	update_session(session);
 
 	trx_search_latch_release_if_reserved(prebuilt->trx);
 	trx = prebuilt->trx;
@@ -1107,6 +1113,7 @@
 int
 ha_innobase::final_drop_index(
 /*==========================*/
+                              Session *session,
 	Table*	)		/*!< in: Table where indexes are dropped */
 {
 	dict_index_t*	index;		/*!< Index to be dropped */
@@ -1117,7 +1124,7 @@
 		return(HA_ERR_WRONG_COMMAND);
 	}
 
-	update_session();
+	update_session(session);
 
 	trx_search_latch_release_if_reserved(prebuilt->trx);
 	trx_start_if_not_started(prebuilt->trx);
@@ -1198,3 +1205,4 @@
 
 	return(err);
 }
+#endif

=== modified file 'tests/r/auto_increment.result'
--- tests/r/auto_increment.result	2010-06-02 06:49:38 +0000
+++ tests/r/auto_increment.result	2010-06-07 06:04:12 +0000
@@ -222,7 +222,7 @@
 create table t1 (a bigint);
 insert into t1 values (1), (2), (3), (0), (0);
 alter table t1 modify a bigint not null auto_increment primary key;
-ERROR 23000: ALTER TABLE causes auto_increment resequencing, resulting in duplicate entry '0' for key 'PRIMARY'
+ERROR 23000: Duplicate entry '0' for key 'PRIMARY'
 select * from t1;
 a
 1

